from privacyidea.lib.applications import MachineApplicationBase


class MachineApplication(MachineApplicationBase):
    '''
    This is the application for SSH.
    '''
    application_name = "ssh"
    
    pass
